program LU_solv

implicit none
! Variabili in Input
integer :: n
real(kind=8), allocatable :: L(:,:),U(:,:),b(:)
! Variabili locali
integer                   :: i,j,info
real(kind=8), allocatable :: x(:),y(:)

! Apre i file di Input e Output
open(1,file='dati.dat')
open(2,file='d6.ris')

! Legge la dimensione
read(1,*) n
! Alloca la memoria necessaria
allocate(L(n,n),U(n,n),b(n),x(n),y(n),stat=info)
if (info .ne. 0) then
   write(2,*) 'Errore allocazione'
   stop
endif
! Legge L (solo la parte bassa piu' la diagonale)
do i = 1,n
   read(1,*) (L(i,j),j=1,i)
enddo
! Legge U (solo la parte strettamente alta)
do i = 1,n
   read(1,*) (U(i,j),j=i+1,n)
enddo
! Legge b
read(1,*) (b(i),i=1,n)

! Stampa dei dati letti
write(2,*) 'Dimensione:',n
write(2,*) 'Matrice L:'
do i = 1,n
   write(2,'(20f13.4)') (L(i,j),j=1,i),(0.d0,j=i+1,n)
enddo
write(2,*) 'Matrice U:'
do i = 1,n
   write(2,'(20f13.4)') (0.d0,j=1,i-1),1.d0,(U(i,j),j=i+1,n)
enddo
write(2,*) 'Termine noto b:'
write(2,'(f13.4)') (b(i),i=1,n)
! Salta una riga per separare i risultati
write(2,*)

! Risolve Ly = b
call L_solv(n,n,L,b,y)

! Risolve Ux = y
call U_solv(n,n,U,y,x)

! Stampa x
write(2,*) 'Soluzione x:'
write(2,'(f13.4)') (x(i),i=1,n)

! Dealloca gli array
deallocate(L,U,b,x,y,stat=info)
if (info .ne. 0) then
   write(2,*) 'Errore deallocazione'
   stop
endif

! Chiude le unita' di Output
close(1)
close(2)

end program LU_solv

!-----------------------------------------------------------------------------------------

subroutine L_solv(n,nmax,L,rhs,sol)
! Risolve un sistema con matrice triangolare bassa
implicit none

! Variabili in Input
integer, intent(in)       :: n,nmax
real(kind=8), intent(in)  :: L(nmax,n),rhs(n)
! Variabili in Output
real(kind=8), intent(out) :: sol(n)
! Variabili locali
integer                   :: i,j
real(kind=8)              :: som

do i = 1,n
   som = 0.d0
   do j = 1,i-1
      som = som + L(i,j)*sol(j)
   enddo
   sol(i) = (rhs(i) - som) / L(i,i)
enddo

end subroutine L_solv

!-----------------------------------------------------------------------------------------

subroutine U_solv(n,nmax,U,rhs,sol)
! Risolve un sistema con matrice triangolare alta e unitaria
implicit none

! Variabili in Input
integer, intent(in)       :: n,nmax
real(kind=8), intent(in)  :: U(nmax,n),rhs(n)
! Variabili in Output
real(kind=8), intent(out) :: sol(n)
! Variabili locali
integer                   :: i,j
real(kind=8)              :: som

do i = n,1,-1
   som = 0.d0
   do j = i+1,n    ! In alternativa va bene anche: do j = n,i+1,-1
      som = som + U(i,j)*sol(j)
   enddo
   sol(i) = rhs(i) - som
enddo

end subroutine U_solv
