module class_Videoteca

implicit none

! Definizione del nodo tipico della lista
type Nodo_Video
   character(len=100)        :: titolo
   character(len=100)        :: regista
   integer                   :: anno
   integer                   :: noleggi
   type(Nodo_Video), pointer :: prossimo
end type

type Videoteca
   type(Nodo_Video), pointer :: pt_primo
end type

contains

! Supponiamo che contenga gia' le procedure INSERISCI, CERCA e CANCELLA e
! assumiamo che al momento dell'inserimento il contatore dei noleggi venga
! azzerato

!-----------------------------------------------------------------------------------------

   subroutine Stampa_Anno(anno_in,L_Video_in)
   ! Procedura per stampare i film usciti in un determinato anno
   ! Questa Subroutine sara' chiamata dal programma principale come:
   ! call Stampa_Anno(1987,Lista_videoteca)
   implicit none
   ! Variabili di scambio
   integer, intent(in)         :: anno_in
   type(Videoteca), intent(in) :: L_Video_in
   ! Variabili locali
   type(Nodo_Video), pointer   :: video_corrente

   ! Vai al primo nodo ("pieno")
   video_corrente => L_Video_in%pt_primo%prossimo
   do while ( associated(video_corrente) )
      ! Controlla anno di uscita
      if (video_corrente%anno .eq. anno_in) then
         write(*,*) video_corrente%titolo,' by ',video_corrente%regista
      endif
      ! Passa al prossimo
      video_corrente => video_corrente%prossimo
   enddo

   end subroutine Stampa_Anno

!-----------------------------------------------------------------------------------------

   recursive subroutine RStampa_Anno(anno_in,Nodo_Video_in)
   ! Versione ricorsiva della procedura per stampare i film usciti in un determinato anno
   ! Questa Subroutine sara' chiamata dal programma principale come:
   ! call RStampa_Anno(1987,Lista_videoteca%pt_primo%prossimo)
   implicit none
   ! Variabili di scambio
   integer, intent(in)                   :: anno_in
   type(Nodo_Video), pointer, intent(in) :: Nodo_Video_in

   if ( associated(Nodo_Video_in) ) then
      ! Controlla anno di uscita
      if (Nodo_Video_in%anno .eq. anno_in) then
         write(*,*) Nodo_Video_in%titolo,' by ',Nodo_Video_in%regista
      endif
      ! CHIAMATA RICORSIVA: passo al prossimo nodo
      call RStampa_Anno(anno_in,Nodo_Video_in%prossimo)
   else
      ! BASE: nodo vuoto, esco
      return
   endif

   end subroutine RStampa_Anno

!-----------------------------------------------------------------------------------------

   subroutine Incremente_Noleggi(titolo_in,L_Video_in)
   ! Procedura per incrementare di una unita' il film dal titolo "titolo_in".
   ! Questa Subroutine sara' chiamata dal programma principale come:
   ! call Incremente_Noleggi('Rambo',Lista_videoteca)
   implicit none
   ! Variabili di scambio
   integer, intent(in)         :: titolo_in
   type(Videoteca), intent(in) :: L_Video_in
   ! Variabili locali
   type(Nodo_Video), pointer   :: video_corrente

   ! Vai al primo nodo ("pieno")
   video_corrente => L_Video_in%pt_primo%prossimo
   do while ( associated(video_corrente) )
      ! Controlla titolo
      if (video_corrente%titolo .eq. titolo_in) then
         video_corrente%noleggi = video_corrente%noleggi + 1
         ! Posso uscire
         return
      endif
      ! Passa al prossimo
      video_corrente => video_corrente%prossimo
   enddo

   end subroutine Incremente_Noleggi

!-----------------------------------------------------------------------------------------

   subroutine Stampa_Anno_Alfabetico(anno_in,L_Video_in)
   ! Procedura per stampare i film usciti in un determinato anno in ordine
   ! alfabetico.
   ! Questa Subroutine sara' chiamata dal programma principale come:
   ! call Stampa_Anno_Alfabetico(1987,Lista_videoteca)
   implicit none
   ! Variabili di scambio
   integer, intent(in)         :: anno_in
   type(Videoteca), intent(in) :: L_Video_in
   ! Variabili locali
   type(Nodo_Video), pointer   :: video_corrente
   type(Videoteca)             :: film_ordine_alfabetico

   ! Inizializza la videoteca ordinata (uso la funzione Nuova_Videoteca per la
   ! creazione di una nuova lista)
   film_ordine_alfabetico = Nuova_Videoteca()

   ! Vai al primo nodo ("pieno")
   video_corrente => L_Video_in%pt_primo%prossimo
   do while ( associated(video_corrente) )
      ! Controlla anno di uscita
      if (video_corrente%anno .eq. anno_in) then
         ! Inserisci questo nodo nella lista ordinata
         call Inserisci_Ordinata(video_corrente,film_ordine_alfabetico%pt_primo%prossimo)
      endif
      ! Passa al prossimo
      video_corrente => video_corrente%prossimo
   enddo

   ! Stampa la lista ordinata
   video_corrente => film_ordine_alfabetico%pt_primo%prossimo
   do while ( associated(video_corrente) )
      write(*,*) video_corrente%titolo,' by ',video_corrente%regista
      ! Passa al prossimo
      video_corrente => video_corrente%prossimo
   enddo

   end subroutine Stampa_Anno_Alfabetico

!-----------------------------------------------------------------------------------------

   recursive subroutine Inserisci_Ordinata(Nodo_Video_in,Nodo_Lista_Ordinata)
   ! Inserisce un video in una lista in cui i film sono tutti in ordine alfabetico
   implicit none
   ! Variabili di scambio
   type(Nodo_Video), pointer, intent(in)    :: Nodo_Video_in
   type(Nodo_Video), pointer, intent(inout) :: Nodo_Lista_Ordinata
   ! Variaboli locali
   type(Nodo_Video), pointer, intent(inout) :: Nodo_Nuovo

   if ( .not. associated(Nodo_Lista_Ordinata) ) then
      ! Se il nodo e' vuoto, allora aggiungo il mio video qui
      ! Alloco lo spazio necessario
      allocate(Nodo_Nuovo)
      ! Copio le informazioni del Nodo_Video_in
      Nodo_Nuovo%titolo   = Nodo_Video_in%titolo
      Nodo_Nuovo%regista  = Nodo_Video_in%regista
      Nodo_Nuovo%anno     = Nodo_Video_in%anno
      Nodo_Nuovo%noleggi  = Nodo_Video_in%noleggi
      ! Annullo il puntatore al prossimo elemento
      Nodo_Nuovo%prossimo => null()
      ! Faccio puntare il nodo della lista ordinata al nuovo nodo
      Nodo_Lista_Ordinata => Nodo_Nuovo
   elseif (Nodo_Video_in%titolo .lt. Nodo_Lista_Ordinata%titolo) then
      ! Il titolo di questo film e' minore di quello in lista, lo inserisco qui
      ! Alloco lo spazio necessario
      allocate(Nodo_Nuovo)
      ! Copio le informazioni del Nodo_Video_in
      Nodo_Nuovo%titolo   = Nodo_Video_in%titolo
      Nodo_Nuovo%regista  = Nodo_Video_in%regista
      Nodo_Nuovo%anno     = Nodo_Video_in%anno
      Nodo_Nuovo%noleggi  = Nodo_Video_in%noleggi
      Nodo_Nuovo%prossimo => Nodo_Lista_Ordinata
      Nodo_Lista_Ordinata%prossimo => Nodo_Nuovo
   else
      ! CHIAMATA RICORSIVA: passo al prossimo nodo della lista ordinata
      call Inserisci_Ordinata(Nodo_Video_in,Nodo_Lista_Ordinata%prossimo)
   endif

   end subroutine Inserisci_Ordinata

!-----------------------------------------------------------------------------------------

end module class_Videoteca
